@@ -30,9 +30,11 @@
 #include <QtCore/QDir>
 #include <QtCore/QFile>
 #include <QtCore/QTimer>
+#include <QtCore/QCommandLineParser>
 #include <QApplication>
 
 #include <QDBusConnection>
+#include <QDBusConnectionInterface>
 #include <QDBusInterface>
 #include <QDBusServiceWatcher>
 #include <QDBusPendingCall>
@@ -46,16 +48,24 @@
 #include <KPluginInfo>
 #include <KPluginMetaData>
 
-Q_DECLARE_LOGGING_CATEGORY(KDED);
-Q_LOGGING_CATEGORY(KDED, "kf5.kded");
+#ifdef Q_OS_OSX
+#include <CoreFoundation/CoreFoundation.h>
+#endif
 
-Kded *Kded::_self = 0;
+Q_DECLARE_LOGGING_CATEGORY(KDED)
+
+#if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
+// logging category for this framework, default: log stuff >= warning
+Q_LOGGING_CATEGORY(KDED, "kf5.kded", QtWarningMsg)
+#else
+Q_LOGGING_CATEGORY(KDED, "kf5.kded")
+#endif
+
+Kded *Kded::_self = nullptr;
 
-static bool checkStamps;
 static bool delayedCheck;
 static bool bCheckSycoca;
 static bool bCheckUpdates;
-static bool s_checkStampsDefault = true;
 
 #ifdef Q_DBUS_EXPORT
 extern Q_DBUS_EXPORT void qDBusAddSpyHook(void (*)(const QDBusMessage &));
@@ -63,44 +73,15 @@ extern Q_DBUS_EXPORT void qDBusAddSpyHook(void (*)(const QDBusMessage &));
 extern QDBUS_EXPORT void qDBusAddSpyHook(void (*)(const QDBusMessage &));
 #endif
 
-static void runBuildSycoca(QObject *callBackObj = 0, const char *callBackSlot = 0)
-{
-    const QString exe = QStandardPaths::findExecutable(KBUILDSYCOCA_EXENAME);
-    Q_ASSERT(!exe.isEmpty());
-    QStringList args;
-    if (QStandardPaths::isTestModeEnabled()) {
-        args.append("--testmode");
-    }
-    if (checkStamps) {
-        args.append("--checkstamps");
-    }
-    if (delayedCheck) {
-        args.append("--nocheckfiles");
-    } else {
-        checkStamps = false;    // useful only during kded startup
-    }
-    if (callBackObj) {
-        QVariantList argList;
-        argList << exe << args << QStringList() << QString();
-        KToolInvocation::ensureKdeinitRunning();
-        QDBusInterface klauncher(QStringLiteral("org.kde.klauncher5"), QStringLiteral("/KLauncher"), QStringLiteral("org.kde.KLauncher"), QDBusConnection::sessionBus());
-        klauncher.callWithCallback("kdeinit_exec_wait", argList, callBackObj, callBackSlot);
-    } else {
-        KToolInvocation::kdeinitExecWait(exe, args);
-    }
-}
-
 static void runKonfUpdate()
 {
     KToolInvocation::kdeinitExecWait(QStringLiteral(KCONF_UPDATE_EXE),
-            QStringList(), 0, 0, "0" /*no startup notification*/);
+            QStringList(), nullptr, nullptr, "0" /*no startup notification*/);
 }
 
 Kded::Kded()
-    : m_pDirWatch(0)
+    : m_pDirWatch(nullptr)
     , m_pTimer(new QTimer(this))
-    , m_recreateCount(0)
-    , m_recreateBusy(false)
     , m_needDelayedCheck(false)
 {
     _self = this;
@@ -126,7 +107,7 @@ Kded::Kded()
 
 Kded::~Kded()
 {
-    _self = 0;
+    _self = nullptr;
     m_pTimer->stop();
     delete m_pTimer;
     delete m_pDirWatch;
@@ -160,32 +141,28 @@ void Kded::messageFilter(const QDBusMessage &message)
         return;
     }
 
-    if (self()->m_dontLoad.value(obj, 0)) {
+    if (self()->m_dontLoad.value(obj, nullptr)) {
         return;
     }
 
-    KDEDModule *module = self()->loadModule(obj, true);
-    if (!module) {
-        qCWarning(KDED) << "Failed to load module for " << obj;
-    }
-    Q_UNUSED(module);
+    self()->loadModule(obj, true);
 }
 
 static int phaseForModule(const KPluginMetaData &module)
 {
-    const QVariant phasev = module.rawData().value("X-KDE-Kded-phase").toVariant();
+    const QVariant phasev = module.rawData().value(QStringLiteral("X-KDE-Kded-phase")).toVariant();
     return phasev.isValid() ? phasev.toInt() : 2;
 }
 
 QVector<KPluginMetaData> Kded::availableModules() const
 {
-    QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins("kf5/kded");
+    QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins(QStringLiteral("kf5/kded"));
     QSet<QString> moduleIds;
     foreach (const KPluginMetaData &md, plugins) {
         moduleIds.insert(md.pluginId());
     }
     // also search for old .desktop based kded modules
-    KPluginInfo::List oldStylePlugins = KPluginInfo::fromServices(KServiceTypeTrader::self()->query("KDEDModule"));
+    KPluginInfo::List oldStylePlugins = KPluginInfo::fromServices(KServiceTypeTrader::self()->query(QStringLiteral("KDEDModule")));
     foreach (const KPluginInfo &info, oldStylePlugins) {
         if (moduleIds.contains(info.pluginName())) {
             qCWarning(KDED).nospace() << "kded module " << info.pluginName() << " has already been found using "
@@ -219,7 +196,14 @@ static KPluginMetaData findModule(const QString &id)
 void Kded::initModules()
 {
     m_dontLoad.clear();
-    bool kde_running = !qgetenv("KDE_FULL_SESSION").isEmpty();
+#ifdef Q_OS_OSX
+    // it seems there is no reason to honour KDE_FULL_SESSION on OS X even if it's set for some reason.
+    // That way kdeinit5 is always auto-started if required (and the kded is as good a candidate
+    // for starting this service as any other application).
+    bool kde_running = false;
+#else
+    bool kde_running = !qEnvironmentVariableIsEmpty("KDE_FULL_SESSION");
+#endif
     if (kde_running) {
 #ifndef Q_OS_WIN
         // not the same user like the one running the session (most likely we're run via sudo or something)
@@ -247,6 +231,9 @@ void Kded::initModules()
     foreach (const KPluginMetaData &module, kdedModules) {
         // Should the service load on startup?
         const bool autoload = isModuleAutoloaded(module);
+        if (!platformSupportsModule(module)) {
+            continue;
+        }
 
         // see ksmserver's README for description of the phases
         bool prevent_autoload = false;
@@ -303,7 +290,7 @@ void Kded::loadSecondPhase()
 
 void Kded::noDemandLoad(const QString &obj)
 {
-    m_dontLoad.insert(obj.toLatin1(), this);
+    m_dontLoad.insert(obj, this);
 }
 
 void Kded::setModuleAutoloading(const QString &obj, bool autoload)
@@ -314,7 +301,7 @@ void Kded::setModuleAutoloading(const QString &obj, bool autoload)
     if (!module.isValid()) {
         return;
     }
-    KConfigGroup cg(config, QString("Module-").append(module.pluginId()));
+    KConfigGroup cg(config, QStringLiteral("Module-").append(module.pluginId()));
     cg.writeEntry("autoload", autoload);
     cg.sync();
 }
@@ -331,11 +318,18 @@ bool Kded::isModuleAutoloaded(const KPluginMetaData &module) const
     }
     KSharedConfig::Ptr config = KSharedConfig::openConfig();
     bool autoload = module.rawData().value(QStringLiteral("X-KDE-Kded-autoload")).toVariant().toBool();
-    KConfigGroup cg(config, QString("Module-").append(module.pluginId()));
+    KConfigGroup cg(config, QStringLiteral("Module-").append(module.pluginId()));
     autoload = cg.readEntry("autoload", autoload);
     return autoload;
 }
 
+bool Kded::platformSupportsModule(const KPluginMetaData &module) const
+{
+    const QStringList supportedPlatforms = KPluginMetaData::readStringList(module.rawData(), QStringLiteral("X-KDE-OnlyShowOnQtPlatforms"));
+
+    return supportedPlatforms.isEmpty() || supportedPlatforms.contains(qApp->platformName());
+}
+
 bool Kded::isModuleLoadedOnDemand(const QString &obj) const
 {
     return isModuleLoadedOnDemand(findModule(obj));
@@ -359,11 +353,11 @@ bool Kded::isModuleLoadedOnDemand(const KPluginMetaData &module) const
 KDEDModule *Kded::loadModule(const QString &obj, bool onDemand)
 {
     // Make sure this method is only called with valid module names.
-    if (obj.contains('/')) {
+    if (obj.contains(QLatin1Char('/'))) {
         qCWarning(KDED) << "attempting to load invalid kded module name:" << obj;
         return nullptr;
     }
-    KDEDModule *module = m_modules.value(obj, 0);
+    KDEDModule *module = m_modules.value(obj, nullptr);
     if (module) {
         return module;
     }
@@ -377,7 +371,7 @@ KDEDModule *Kded::loadModule(const KPluginMetaData &module, bool onDemand)
         return nullptr;
     }
     const QString moduleId = module.pluginId();
-    KDEDModule *oldModule = m_modules.value(moduleId, 0);
+    KDEDModule *oldModule = m_modules.value(moduleId, nullptr);
     if (oldModule) {
         qCDebug(KDED) << "kded module" << moduleId << "is already loaded.";
         return oldModule;
@@ -400,7 +394,7 @@ KDEDModule *Kded::loadModule(const KPluginMetaData &module, bool onDemand)
         kdedModule = factory->create<KDEDModule>(this);
     } else {
         // TODO: remove this fallback code, the kded modules should all be fixed instead
-        KPluginLoader loader2("kded_" + module.fileName());
+        KPluginLoader loader2(QStringLiteral("kded_") + module.fileName());
         factory = loader2.factory();
         if (factory) {
             qCWarning(KDED).nospace() << "found kded module " << moduleId
@@ -425,7 +419,7 @@ KDEDModule *Kded::loadModule(const KPluginMetaData &module, bool onDemand)
 
 bool Kded::unloadModule(const QString &obj)
 {
-    KDEDModule *module = m_modules.value(obj, 0);
+    KDEDModule *module = m_modules.value(obj, nullptr);
     if (!module) {
         return false;
     }
@@ -443,9 +437,6 @@ QStringList Kded::loadedModules()
 void Kded::slotKDEDModuleRemoved(KDEDModule *module)
 {
     m_modules.remove(module->moduleName());
-    //KLibrary *lib = m_libs.take(module->moduleName());
-    //if (lib)
-    //   lib->unload();
 }
 
 void Kded::slotApplicationRemoved(const QString &name)
@@ -531,19 +522,19 @@ void Kded::runDelayedCheck()
 
 void Kded::recreate(bool initial)
 {
-    m_recreateBusy = true;
     // Using KLauncher here is difficult since we might not have a
     // database
 
     if (!initial) {
         updateDirWatch(); // Update tree first, to be sure to miss nothing.
-        runBuildSycoca(this, SLOT(recreateDone()));
+        KSycoca::self()->ensureCacheValid();
+        recreateDone();
     } else {
         if (!delayedCheck) {
             updateDirWatch();    // this would search all the directories
         }
         if (bCheckSycoca) {
-            runBuildSycoca();
+            KSycoca::self()->ensureCacheValid();
         }
         recreateDone();
         if (delayedCheck) {
@@ -561,19 +552,7 @@ void Kded::recreateDone()
 {
     updateResourceList();
 
-    for (; m_recreateCount; m_recreateCount--) {
-        QDBusMessage msg = m_recreateRequests.takeFirst();
-        QDBusConnection::sessionBus().send(msg.createReply());
-    }
-    m_recreateBusy = false;
-
-    // Did a new request come in while building?
-    if (!m_recreateRequests.isEmpty()) {
-        m_pTimer->start(2000);
-        m_recreateCount = m_recreateRequests.count();
-    } else {
-        initModules();
-    }
+    initModules();
 }
 
 void Kded::dirDeleted(const QString &path)
@@ -583,23 +562,7 @@ void Kded::dirDeleted(const QString &path)
 
 void Kded::update(const QString &)
 {
-    if (!m_recreateBusy) {
-        m_pTimer->start(10000);
-    }
-}
-
-void Kded::recreate(const QDBusMessage &msg)
-{
-    if (!m_recreateBusy) {
-        if (m_recreateRequests.isEmpty()) {
-            m_pTimer->start(0);
-            m_recreateCount = 0;
-        }
-        m_recreateCount++;
-    }
-    msg.setDelayedReply(true);
-    m_recreateRequests.append(msg);
-    return;
+    m_pTimer->start(10000);
 }
 
 void Kded::readDirectory(const QString &_path)
@@ -615,49 +578,8 @@ void Kded::readDirectory(const QString &_path)
 
     Q_ASSERT(path != QDir::homePath());
     m_pDirWatch->addDir(path, KDirWatch::WatchFiles | KDirWatch::WatchSubDirs);       // add watch on this dir
-    return; // KDirWatch now claims to also support recursive watching
-#if 0
-    QDir d(_path, QString(), QDir::Unsorted, QDir::Readable | QDir::Executable | QDir::Dirs | QDir::Hidden);
-    // set QDir ...
-
-    //************************************************************************
-    //                           Setting dirs
-    //************************************************************************
-
-    if (!d.exists()) {                            // exists&isdir?
-        qCDebug(KDED) << "Does not exist:" << _path;
-        return;                             // return false
-    }
-
-    // Note: If some directory is gone, dirwatch will delete it from the list.
-
-    //************************************************************************
-    //                               Reading
-    //************************************************************************
-    QString file;
-    unsigned int i;                           // counter and string length.
-    unsigned int count = d.count();
-    for (i = 0; i < count; i++) {                       // check all entries
-        if (d[i] == "." || d[i] == ".." || d[i] == "magic") {
-            continue;    // discard those ".", "..", "magic"...
-        }
-
-        file = path;                           // set full path
-        file += d[i];                          // and add the file name.
-
-        readDirectory(file);        // yes, dive into it.
-    }
-#endif
 }
 
-/*
-bool Kded::isWindowRegistered(long windowId) const
-{
-  return m_globalWindowIdList.contains(windowId);
-
-}
-*/
-
 void Kded::registerWindowId(qlonglong windowId, const QString &sender)
 {
     if (!m_windowIdList.contains(sender)) {
@@ -711,14 +633,14 @@ KUpdateD::KUpdateD()
     QObject::connect(m_pDirWatch, &KDirWatch::dirty,
                      this, &KUpdateD::slotNewUpdateFile);
 
-    const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, "kconf_update", QStandardPaths::LocateDirectory);
+    const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("kconf_update"), QStandardPaths::LocateDirectory);
     for (QStringList::ConstIterator it = dirs.begin();
             it != dirs.end();
             ++it) {
         QString path = *it;
         Q_ASSERT(path != QDir::homePath());
-        if (path[path.length() - 1] != '/') {
-            path += '/';
+        if (path[path.length() - 1] != QLatin1Char('/')) {
+            path += QLatin1Char('/');
         }
 
         if (!m_pDirWatch->contains(path)) {
@@ -748,62 +670,96 @@ KBuildsycocaAdaptor::KBuildsycocaAdaptor(QObject *parent)
 {
 }
 
-void KBuildsycocaAdaptor::recreate(const QDBusMessage &msg)
+void KBuildsycocaAdaptor::recreate()
 {
-    Kded::self()->recreate(msg);
+    Kded::self()->recreate();
 }
 
+// KF6: remove
 bool KBuildsycocaAdaptor::isTestModeEnabled()
 {
     return QStandardPaths::isTestModeEnabled();
 }
 
+// KF6: remove
 void KBuildsycocaAdaptor::enableTestMode()
 {
     QStandardPaths::enableTestMode(true);
 }
 
-static void setupAppInfo(QCoreApplication *app)
+static void setupAppInfo(QApplication *app)
 {
-    app->setApplicationName("kded5");
-    app->setOrganizationDomain("kde.org");
+    app->setApplicationName(QStringLiteral("kded5"));
+    app->setApplicationDisplayName(QStringLiteral("KDE Daemon"));
+    app->setOrganizationDomain(QStringLiteral("kde.org"));
+    app->setApplicationVersion(QStringLiteral(KDED_VERSION_STRING));
 }
 
 extern "C" Q_DECL_EXPORT int kdemain(int argc, char *argv[])
 {
+#ifdef Q_OS_OSX
+    CFBundleRef mainBundle = CFBundleGetMainBundle();
+    if (mainBundle) {
+        // get the application's Info Dictionary. For app bundles this would live in the bundle's Info.plist,
+        // for regular executables it is obtained in another way.
+        CFMutableDictionaryRef infoDict = (CFMutableDictionaryRef) CFBundleGetInfoDictionary(mainBundle);
+        if (infoDict) {
+            // Add or set the "LSUIElement" key with/to value "1". This can simply be a CFString.
+            CFDictionarySetValue(infoDict, CFSTR("LSUIElement"), CFSTR("1"));
+            // That's it. We're now considered as an "agent" by the window server, and thus will have
+            // neither menubar nor presence in the Dock or App Switcher.
+        }
+    }
+#endif
     //options.add("check", qi18n("Check Sycoca database only once"));
 
     // WABA: Make sure not to enable session management.
-    putenv(qstrdup("SESSION_MANAGER="));
+    qunsetenv("SESSION_MANAGER");
+
+    // In older versions, QApplication creation was postponed until after
+    // testing for --check, in which case, only a QCoreApplication was created.
+    // Since that option is no longer used at startup, we removed that speed
+    // optimization for code clarity and easier support of standard parameters.
+    QApplication app(argc, argv);
+    setupAppInfo(&app);
+    app.setQuitOnLastWindowClosed(false);
+
+    QCommandLineParser parser;
+    parser.addHelpOption();
+    parser.addVersionOption();
+    parser.addOption(QCommandLineOption(QStringLiteral("check"), QStringLiteral("Check cache validity")));
+    parser.process(app);
 
     // Parse command line before checking D-Bus
-    if (argc > 1 && QByteArray(argv[1]) == "--check") {
+    if (parser.isSet(QStringLiteral("check"))) {
         // KDBusService not wanted here.
-        QCoreApplication app(argc, argv);
-        setupAppInfo(&app);
-
-        KSharedConfig::Ptr config = KSharedConfig::openConfig();
-        KConfigGroup cg(config, "General");
-        checkStamps = cg.readEntry("CheckFileStamps", s_checkStampsDefault);
-
-        runBuildSycoca();
+        KSycoca::self()->ensureCacheValid();
         runKonfUpdate();
         return 0;
     }
 
-    QApplication app(argc, argv);
-    setupAppInfo(&app);
-    app.setApplicationDisplayName("KDE Daemon");
-    app.setQuitOnLastWindowClosed(false);
-
     KDBusService service(KDBusService::Unique);
 
+    QDBusConnectionInterface *bus = QDBusConnection::sessionBus().interface();
+    // Also register as all the names we should respond to (org.kde.kcookiejar, org.kde.khotkeys etc.)
+    // so that the calling code is independent from the physical "location" of the service.
+    const QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins(QStringLiteral("kf5/kded"));
+    foreach (const KPluginMetaData &metaData, plugins) {
+        const QString serviceName = metaData.rawData().value(QStringLiteral("X-KDE-DBus-ServiceName")).toString();
+        if (serviceName.isEmpty()) {
+            qCWarning(KDED) << "No X-KDE-DBus-ServiceName found in" << metaData.fileName();
+            continue;
+        }
+        if (!bus->registerService(serviceName)) {
+            qCWarning(KDED) << "Couldn't register name" << serviceName << "with DBUS - another process owns it already!";
+        }
+    }
+
     KSharedConfig::Ptr config = KSharedConfig::openConfig();
     KConfigGroup cg(config, "General");
 
     bCheckSycoca = cg.readEntry("CheckSycoca", true);
     bCheckUpdates = cg.readEntry("CheckUpdates", false);
-    checkStamps = cg.readEntry("CheckFileStamps", s_checkStampsDefault);
     delayedCheck = cg.readEntry("DelayedCheck", false);
 
 #ifndef Q_OS_WIN
@@ -822,7 +778,7 @@ extern "C" Q_DECL_EXPORT int kdemain(int argc, char *argv[])
     }
 
 //NOTE: We are going to change how KDE starts and this certanly doesn't fit on the new design.
-#ifdef Q_OS_LINUX
+#if defined Q_OS_LINUX || defined Q_OS_CYGWIN
     // Tell KSplash that KDED has started
     QDBusMessage ksplashProgressMessage = QDBusMessage::createMethodCall(QStringLiteral("org.kde.KSplash"),
                                           QStringLiteral("/KSplash"),
@@ -834,7 +790,7 @@ extern "C" Q_DECL_EXPORT int kdemain(int argc, char *argv[])
 
     runKonfUpdate(); // Run it once.
 
-#ifdef Q_OS_LINUX
+#if defined Q_OS_LINUX || defined Q_OS_CYGWIN
     ksplashProgressMessage = QDBusMessage::createMethodCall(QStringLiteral("org.kde.KSplash"),
                              QStringLiteral("/KSplash"),
                              QStringLiteral("org.kde.KSplash"),
